@@ -1,9 +1,9 @@
-/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
- * applicable.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -109,7 +109,7 @@ static void create_testfile(apr_pool_t *p, const char *fname)
 
     printf("Creating a test file...\n");
     rv = apr_file_open(&f, fname, 
-                 APR_CREATE | APR_WRITE | APR_TRUNCATE | APR_BUFFERED,
+                 APR_FOPEN_CREATE | APR_FOPEN_WRITE | APR_FOPEN_TRUNCATE | APR_FOPEN_BUFFERED,
                  APR_UREAD | APR_UWRITE, p);
     if (rv) {
         fprintf(stderr, "apr_file_open()->%d/%s\n",
@@ -147,7 +147,7 @@ static void create_testfile(apr_pool_t *p, const char *fname)
     }
 
     rv = apr_stat(&finfo, fname, APR_FINFO_NORM, p);
-    if (rv != APR_SUCCESS && rv != APR_INCOMPLETE) {
+    if (rv != APR_SUCCESS && ! APR_STATUS_IS_INCOMPLETE(rv)) {
         fprintf(stderr, "apr_stat()->%d/%s\n",
                 rv, apr_strerror(rv, buf, sizeof buf));
         exit(1);
@@ -188,7 +188,7 @@ static int client(client_socket_mode_t socket_mode, char *host)
     apr_setup(&p, &sock, &family);
     create_testfile(p, TESTFILE);
 
-    rv = apr_file_open(&f, TESTFILE, APR_READ, 0, p);
+    rv = apr_file_open(&f, TESTFILE, APR_FOPEN_READ, 0, p);
     if (rv != APR_SUCCESS) {
         fprintf(stderr, "apr_file_open()->%d/%s\n",
                 rv,
